#!/usr/bin/python
# -*- coding: utf-8 -*-

# =======================================================================================
#
#  Copyright 2019 Sophos Limited. All rights reserved.
#
#  Sophos and Sophos Anti-Virus are registered trademarks of Sophos Limited.
#
#  All other product and company names mentioned are trademarks or registered
#  trademarks of their respective owners.
#
# =======================================================================================

import argparse
import select
import socket
import sys

# =======================================================================================

TARGET_PORT = 54445
MESSAGE     = b'SOPHOS ENDPOINT IPS TEST PATTERN\x00'

# =======================================================================================

def be_server(server_address):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    if not sock:
        print('failed to create socket')
        sys.exit(-1)
    try:
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.bind((server_address, TARGET_PORT))
        sock.listen(10)
        sock.setblocking(False)
    except Exception as e: 
        print('exception from socket bind/listen:')
        print(e)
        sys.exit(-1)
    try:
        where = sock.getsockname()
        print('TCP server listening on {}:{}'.format(where[0], where[1]))
        while True:
            (readable, _, _) = select.select([sock], [], [], 0.2)
            if sock in readable:
                (conn,addr) = sock.accept()
                print('received connection from {}:{}'.format(addr[0], addr[1]))
                conn.close()
    except KeyboardInterrupt:
        print('stopping TCP server')
    except Exception as e: 
        print('exception from socket accept:')
        print(e)
        sys.exit(-1)
    
# =======================================================================================

def send_tcp_test_pattern(address):
    print('sending TCP test pattern to {}:{}'.format(address, TARGET_PORT))
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    if not sock:
        print('failed to create socket')
        sys.exit(-1)
    try:
        sock.connect((address, TARGET_PORT))
        sock.send(MESSAGE)
    except Exception as e: 
        print('exception sending test pattern to {}:{}'.format(address, TARGET_PORT))
        print(e)
        sys.exit(-1)
        
# =======================================================================================

def send_udp_test_pattern(address):
    print('sending UDP test pattern to {}:{}'.format(address, TARGET_PORT))
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    if not sock:
        print('failed to create socket')
        sys.exit(-1)
    try:
        sock.sendto(MESSAGE, (address, TARGET_PORT))
    except Exception as e: 
        print('exception sending test pattern to {}:{}'.format(address, TARGET_PORT))
        print(e)
        sys.exit(-1)
    
# =======================================================================================

def send_icmp_test_pattern(address):
    print('sending ICMP test pattern to {}:{}'.format(address, TARGET_PORT))
    icmp_header = b'\x08\x00\x5c\x85\x00\x01\x00\x05'
    icmp_message = icmp_header + MESSAGE
    sock = socket.socket(socket.AF_INET, socket.SOCK_RAW, socket.IPPROTO_ICMP)
    if not sock:
        print('failed to create socket')
        sys.exit(-1)
    try:
        sock.sendto(icmp_message, (address, TARGET_PORT))
    except Exception as e: 
        print('exception sending test pattern to {}:{}'.format(address, TARGET_PORT))
        print(e)
        sys.exit(-1)
    
# =======================================================================================

parser = argparse.ArgumentParser(description='Sophos Endpoint IPS test tool')

parser.add_argument('--server', '-s',
                    help='run as a TCP server, to receive the test pattern',
                    action='store_true')

parser.add_argument('--type', '-t',
                    help='network packet type',
                    choices=['icmp', 'udp', 'tcp'],
                    default='udp')

parser.add_argument('--address', '-a',
                    help='network address of the server',
                    required=False)

args = parser.parse_args()

if args.server:
    # server mode
    server_address = socket.gethostbyname(socket.gethostname())
    if args.address:
        server_address = args.address
    be_server(server_address)
else:
    # client mode
    target_server = 'ipstest.sophostest.com'
    if args.address:
        target_server = args.address
    if args.type == 'tcp':
        send_tcp_test_pattern(target_server)
    elif args.type == 'udp':
        send_udp_test_pattern(target_server)
    elif args.type == 'icmp':
        send_icmp_test_pattern(target_server)

sys.exit(0)
